unsigned int		RATE;

int			device_open();
void			device_start(unsigned int,int,int);
void			device_play(unsigned char*,int);
void			device_volume(int);
void			device_stop();
void			device_wait();
void			device_close();


/* ********************************************	*/
/* 	NEWS-OS ɥǥХ		*/
/* ********************************************	*/
#ifdef NEWS_CHECK

#include <newsiodev/sound.h>

// about NEWS3000. NEWS5900 is MAX = (16), MIN = (-16)

#define SONY_MAX_VOL    (0)           
#define SONY_MIN_VOL    (-32)

int			SBFD;
struct sbparam		SBP;
struct sblevel		SBL;

int device_open()
{
   if ((SBFD = open("/dev/sb0",O_WRONLY | O_NDELAY)) < 0)
     {
     printf("Can't open Sound Device.\n");
     return 1;
     };
   printf("NEWS-OS SOUND DEVICE opened.\n");
   return 0;
};

void device_start(unsigned int rate,int channel,int bit)
{
/* RATE8000,RATE9450,RATE18900,RATE37800 */

   RATE = 37800;

   SBP.sb_mode = STRPCM;
   SBP.sb_compress = 0;		// ߡ
   SBP.sb_rate = RATE;
   SBP.sb_channel = channel;
   SBP.sb_bitwidth = bit;
   ioctl(SBFD,SBIOCSETPARAM,&SBP);
   device_volume(80);

   printf("NEWS-OS SOUND DEVICE started.\n");
};

void device_play(unsigned char data[],int datan)
{
   write(SBFD,data,datan);
};

void device_volume(int v)
{
   int			gain;

   gain = SONY_MIN_VOL + ((SONY_MAX_VOL - SONY_MIN_VOL) * v / 255);
   SBL.sb_left = SBL.sb_right = gain << 16;
   ioctl(SBFD,SBIOCSETOUTLVL,&SBL);
};

void device_stop()
{
   ioctl(SBFD,SBIOCABORT,0);
};

void device_wait()
{
   ioctl(SBFD,SBIOCWAIT,0);
 };  

void device_close()
{
   ioctl(SBFD,SBIOCRESET,0);
   close(SBFD);
};

#endif
/* ******************************************** */
/* ******************************************** */

/* ******************************************** */
/*	SUN-OS 4.1.x ɥǥХ		*/
/*	+ Solaris/SunOS 5.x			*/
/* ******************************************** */
#ifdef SUN_CHECK

#include <fcntl.h>
#include <stropts.h>
#include <sun/audioio.h>
#ifdef SOLARIS_CHANGE
#include <sys/audioio.h>
#else
#include "audioC2U.h"
#endif

int			SBFD,		AUCHK,		BITCHG;
struct audio_info_t	A_INFO;


int device_open()
{
   int			ret;
#ifdef SOLARIS_CHANGE
   audio_device_t	type;
#else
   int			type;
#endif

   if ((SBFD = open("/dev/audio",O_WRONLY | O_NDELAY)) < 0)
     {
     printf("Can't open Sound Device.\n");
     return 1;
     };
   ret = ioctl(SBFD,AUDIO_GETDEV,&type);
#ifdef SOLARIS_CHANGE
   if (ret ||
       (strcmp(type.name, "SUNW,dbri") && strcmp(type.name, "SUNW,CS4231")))
#else
   if (ret || (type == AUDIO_DEV_UNKNOWN) || (type == AUDIO_DEV_AMD))
#endif
     {
     printf("*** SPARC AU AUDIO *** ");
     AUCHK = 1;
     }
   else AUCHK = 0;
   printf("SUN-OS DEVICE opened.\n");
   return 0;
};

void device_start(unsigned int rate,int channel,int bit)
{
   int			ret;

/* 8000, 9600, 11025, 16000, 18900, 22050, 32000, 37800, 44100, 48000 */

   RATE = (AUCHK)?8000:37800;
   AUDIO_INITINFO(&A_INFO);
   A_INFO.play.sample_rate = RATE;
   A_INFO.play.channels    = 1;
   A_INFO.play.precision   = (AUCHK)?8:16;
   A_INFO.play.encoding    = AUDIO_ENCODING_LINEAR;
   A_INFO.play.gain        = 0;
   A_INFO.play.port        = AUDIO_SPEAKER;
   ret = ioctl(SBFD,AUDIO_SETINFO,&A_INFO);
   device_volume(80);

   BITCHG = (bit != 16)?1:0;
   printf("SUN-OS SOUND DEVICE started.\n");
};

void device_play(unsigned char data[],int datan)
{
   int			i;

   if (AUCHK)
     {
     if (!BITCHG)
       {
       for (i=0;i<datan;i+=2) data[i>>1] = Audio_C2U[(unsigned char)data[i]];
       datan >>= 1;
       }
     else
       for (i=0;i<datan;i++) data[i] = Audio_C2U[(unsigned char)data[i]];
     }
   else if (BITCHG)
     {
     for (i=datan;i>=0;i--)
       {
       data[ i<<1   ] = data[i];
       data[(i<<1)|1] = (data[i]&0x80)?0xff:0;
       };
     datan <<= 1;
     };
   write(SBFD,data,datan);
};

void device_volume(int v)
{
   AUDIO_INITINFO(&A_INFO);
   A_INFO.play.gain = AUDIO_MIN_GAIN + 
                    ((AUDIO_MAX_GAIN - AUDIO_MIN_GAIN) * v / 255);
   ioctl(SBFD,AUDIO_SETINFO,&A_INFO);
};
   
void device_stop()
{
   ioctl(SBFD,I_FLUSH,FLUSHW);
};

void device_wait()
{
   do{
     ioctl(SBFD,AUDIO_GETINFO,&A_INFO);
     } while (A_INFO.play.active);
};

void device_close()
{
   close(SBFD);
};

#endif
/* ******************************************** */
/* ******************************************** */

/* ******************************************** */
/*	NONE					*/
/* ******************************************** */
#ifdef NON_CHECK

int                     device_open(){};
void                    device_start(unsigned int,int,int){};
void			device_play(unsigned char*,int){};
void			device_stop() {};
void			device_wait(){};
void			device_close(){};

#endif
/* ******************************************** */
/* ******************************************** */
